/*
 ***********************************************************************
 *                 C++ Wildcard Pattern Matching Library               *
 *                                                                     *
 * Author: Arash Partow (2001)                                         *
 * URL: https://www.partow.net/programming/wildcardmatching/index.html *
 *                                                                     *
 * Copyright notice:                                                   *
 * Free use of the C++ Wildcard Pattern Matching Library is permitted  *
 * under the guidelines and in accordance with the most current        *
 * version of the MIT License.                                         *
 * https://www.opensource.org/licenses/MIT                             *
 *                                                                     *
 ***********************************************************************
*/


#include <cstdio>
#include <string>

#include "globmatch.hpp"


int main()
{
   bool all_valid = true;
   static unsigned int test_id = 0;

   #define test_case(S, P, R)                    \
   {                                             \
      const std::string s(S);                    \
      const std::string p(P);                    \
      test_id++;                                 \
      if (R != glob::match(s, p))                \
      {                                          \
         printf("Failed Test: %d s: %s p: %s\n", \
                test_id,                         \
                s.c_str(),                       \
                p.c_str());                      \
         all_valid = false;                      \
      }                                          \
   }                                             \

   test_case (""            , ""          , true )
   test_case ("a"           , "a*"        , true )
   test_case ("aaa"         , "*aa."      , true )
   test_case ("aaaa"        , "*aa."      , true )
   test_case ("aaaaa"       , "*aa."      , true )
   test_case ("aaaaaa"      , "*aa."      , true )
   test_case ("aaaaabbbbb"  , "*aa.*bb."  , true )
   test_case ("ab"          , "a*"        , true )
   test_case ("ab"          , "a."        , true )
   test_case ("abcd"        , "***d"      , true )
   test_case ("abcd"        , "**cd"      , true )
   test_case ("abcd"        , "*.cd"      , true )
   test_case ("abcd"        , "*b*d"      , true )
   test_case ("abcd"        , "*b.*"      , true )
   test_case ("abcd"        , "*b.."      , true )
   test_case ("abcd"        , "*bc*"      , true )
   test_case ("abcd"        , "*bc."      , true )
   test_case ("abcd"        , "*bcd"      , true )
   test_case ("abcd"        , ".*.d"      , true )
   test_case ("abcd"        , ".*cd"      , true )
   test_case ("abcd"        , "...."      , true )
   test_case ("abcd"        , "...d"      , true )
   test_case ("abcd"        , "..c*"      , true )
   test_case ("abcd"        , "..cd"      , true )
   test_case ("abcd"        , ".b.d"      , true )
   test_case ("abcd"        , ".bc*"      , true )
   test_case ("abcd"        , ".bc."      , true )
   test_case ("abcd"        , ".bcd"      , true )
   test_case ("abcd"        , "a*"        , true )
   test_case ("abcd"        , "a***"      , true )
   test_case ("abcd"        , "a*.d"      , true )
   test_case ("abcd"        , "a*c*"      , true )
   test_case ("abcd"        , "a*cd"      , true )
   test_case ("abcd"        , "a.*d"      , true )
   test_case ("abcd"        , "a..."      , true )
   test_case ("abcd"        , "a..d"      , true )
   test_case ("abcd"        , "a.c."      , true )
   test_case ("abcd"        , "a.cd"      , true )
   test_case ("abcd"        , "ab*"       , true )
   test_case ("abcd"        , "ab**"      , true )
   test_case ("abcd"        , "ab*."      , true )
   test_case ("abcd"        , "ab*d"      , true )
   test_case ("abcd"        , "ab.*"      , true )
   test_case ("abcd"        , "ab.."      , true )
   test_case ("abcd"        , "ab.d"      , true )
   test_case ("abcd"        , "abc*"      , true )
   test_case ("abcd"        , "abc."      , true )
   test_case ("abcd"        , "abcd"      , true )
   test_case ("abcd"        , "abcd*"     , true )
   test_case ("adceb"       , "*a*b"      , true )
   test_case ("xabclabcd"   , "*abcd*"    , true )
   test_case ("xablabcd"    , "*abcd*"    , true )
   test_case ("xalabcd"     , "*abcd*"    , true )
   test_case (""            , "**"        , true )
   test_case (""            , "***"       , true )
   test_case (""            , "****"      , true )
   test_case (""            , "*****"     , true )
   test_case (""            , "******"    , true )
   test_case ("axbxcxd"                                                    , "a*b*c*d"       , true)
   test_case ("axxbxxcxxd"                                                 , "a*b*c*d"       , true)
   test_case ("axxxbxxxcxxxd"                                              , "a*b*c*d"       , true)
   test_case ("axxxxbxxxxcxxxxd"                                           , "a*b*c*d"       , true)
   test_case ("axxxxxbxxxxxcxxxxxd"                                        , "a*b*c*d"       , true)
   test_case ("axxxxxxbxxxxxxcxxxxxxd"                                     , "a*b*c*d"       , true)
   test_case ("axxxxxxxbxxxxxxxcxxxxxxxd"                                  , "a*b*c*d"       , true)
   test_case ("axxxxxxxxbxxxxxxxxcxxxxxxxxd"                               , "a*b*c*d"       , true)
   test_case ("axxxxxxxxxbxxxxxxxxxcxxxxxxxxxd"                            , "a*b*c*d"       , true)
   test_case ("axxxxxxxxxxbxxxxxxxxxxcxxxxxxxxxxd"                         , "a*b*c*d"       , true)
   test_case ("axxxxxxxxxxxbxxxxxxxxxxxcxxxxxxxxxxxd"                      , "a*b*c*d"       , true)
   test_case ("axxxxxxxxxxxxbxxxxxxxxxxxxcxxxxxxxxxxxxd"                   , "a*b*c*d"       , true)
   test_case ("axxxxxxxxxxxxxbxxxxxxxxxxxxxcxxxxxxxxxxxxxd"                , "a*b*c*d"       , true)
   test_case ("axxxxxxxxxxxxxxbxxxxxxxxxxxxxxcxxxxxxxxxxxxxxd"             , "a*b*c*d"       , true)
   test_case ("axxxxxxxxxxxxxxxbxxxxxxxxxxxxxxxcxxxxxxxxxxxxxxxd"          , "a*b*c*d"       , true)
   test_case ("axxxxxxxxxxxxxxxxbxxxxxxxxxxxxxxxxcxxxxxxxxxxxxxxxxd"       , "a*b*c*d"       , true)
   test_case ("axxxxxxxxxxxxxxxxxbxxxxxxxxxxxxxxxxxcxxxxxxxxxxxxxxxxxd"    , "a*b*c*d"       , true)
   test_case ("axxxxxxxxxxxxxxxxxxbxxxxxxxxxxxxxxxxxxcxxxxxxxxxxxxxxxxxxd" , "a*b*c*d"       , true)
   test_case ("axbxcxd"                                                    , "a.*b.*c.*d"    , true)
   test_case ("axxbxxcxxd"                                                 , "a.*b.*c.*d"    , true)
   test_case ("axxxbxxxcxxxd"                                              , "a.*b.*c.*d"    , true)
   test_case ("axxxxbxxxxcxxxxd"                                           , "a.*b.*c.*d"    , true)
   test_case ("axxxxxbxxxxxcxxxxxd"                                        , "a.*b.*c.*d"    , true)
   test_case ("axxxxxxbxxxxxxcxxxxxxd"                                     , "a.*b.*c.*d"    , true)
   test_case ("axxxxxxxbxxxxxxxcxxxxxxxd"                                  , "a.*b.*c.*d"    , true)
   test_case ("axxxxxxxxbxxxxxxxxcxxxxxxxxd"                               , "a.*b.*c.*d"    , true)
   test_case ("axxxxxxxxxbxxxxxxxxxcxxxxxxxxxd"                            , "a.*b.*c.*d"    , true)
   test_case ("axxxxxxxxxxbxxxxxxxxxxcxxxxxxxxxxd"                         , "a.*b.*c.*d"    , true)
   test_case ("axxxxxxxxxxxbxxxxxxxxxxxcxxxxxxxxxxxd"                      , "a.*b.*c.*d"    , true)
   test_case ("axxxxxxxxxxxxbxxxxxxxxxxxxcxxxxxxxxxxxxd"                   , "a.*b.*c.*d"    , true)
   test_case ("axxxxxxxxxxxxxbxxxxxxxxxxxxxcxxxxxxxxxxxxxd"                , "a.*b.*c.*d"    , true)
   test_case ("axxxxxxxxxxxxxxbxxxxxxxxxxxxxxcxxxxxxxxxxxxxxd"             , "a.*b.*c.*d"    , true)
   test_case ("axxxxxxxxxxxxxxxbxxxxxxxxxxxxxxxcxxxxxxxxxxxxxxxd"          , "a.*b.*c.*d"    , true)
   test_case ("axxxxxxxxxxxxxxxxbxxxxxxxxxxxxxxxxcxxxxxxxxxxxxxxxxd"       , "a.*b.*c.*d"    , true)
   test_case ("axxxxxxxxxxxxxxxxxbxxxxxxxxxxxxxxxxxcxxxxxxxxxxxxxxxxxd"    , "a.*b.*c.*d"    , true)
   test_case ("axxxxxxxxxxxxxxxxxxbxxxxxxxxxxxxxxxxxxcxxxxxxxxxxxxxxxxxxd" , "a.*b.*c.*d"    , true)
   test_case ("axbxcxd"                                                    , "a*.b*.c*.d"    , true)
   test_case ("axxbxxcxxd"                                                 , "a*.b*.c*.d"    , true)
   test_case ("axxxbxxxcxxxd"                                              , "a*.b*.c*.d"    , true)
   test_case ("axxxxbxxxxcxxxxd"                                           , "a*.b*.c*.d"    , true)
   test_case ("axxxxxbxxxxxcxxxxxd"                                        , "a*.b*.c*.d"    , true)
   test_case ("axxxxxxbxxxxxxcxxxxxxd"                                     , "a*.b*.c*.d"    , true)
   test_case ("axxxxxxxbxxxxxxxcxxxxxxxd"                                  , "a*.b*.c*.d"    , true)
   test_case ("axxxxxxxxbxxxxxxxxcxxxxxxxxd"                               , "a*.b*.c*.d"    , true)
   test_case ("axxxxxxxxxbxxxxxxxxxcxxxxxxxxxd"                            , "a*.b*.c*.d"    , true)
   test_case ("axxxxxxxxxxbxxxxxxxxxxcxxxxxxxxxxd"                         , "a*.b*.c*.d"    , true)
   test_case ("axxxxxxxxxxxbxxxxxxxxxxxcxxxxxxxxxxxd"                      , "a*.b*.c*.d"    , true)
   test_case ("axxxxxxxxxxxxbxxxxxxxxxxxxcxxxxxxxxxxxxd"                   , "a*.b*.c*.d"    , true)
   test_case ("axxxxxxxxxxxxxbxxxxxxxxxxxxxcxxxxxxxxxxxxxd"                , "a*.b*.c*.d"    , true)
   test_case ("axxxxxxxxxxxxxxbxxxxxxxxxxxxxxcxxxxxxxxxxxxxxd"             , "a*.b*.c*.d"    , true)
   test_case ("axxxxxxxxxxxxxxxbxxxxxxxxxxxxxxxcxxxxxxxxxxxxxxxd"          , "a*.b*.c*.d"    , true)
   test_case ("axxxxxxxxxxxxxxxxbxxxxxxxxxxxxxxxxcxxxxxxxxxxxxxxxxd"       , "a*.b*.c*.d"    , true)
   test_case ("axxxxxxxxxxxxxxxxxbxxxxxxxxxxxxxxxxxcxxxxxxxxxxxxxxxxxd"    , "a*.b*.c*.d"    , true)
   test_case ("axxxxxxxxxxxxxxxxxxbxxxxxxxxxxxxxxxxxxcxxxxxxxxxxxxxxxxxxd" , "a*.b*.c*.d"    , true)
   test_case ("axbxcxd"                                                    , "a.**b.**c.**d" , true)
   test_case ("axxbxxcxxd"                                                 , "a.**b.**c.**d" , true)
   test_case ("axxxbxxxcxxxd"                                              , "a.**b.**c.**d" , true)
   test_case ("axxxxbxxxxcxxxxd"                                           , "a.**b.**c.**d" , true)
   test_case ("axxxxxbxxxxxcxxxxxd"                                        , "a.**b.**c.**d" , true)
   test_case ("axxxxxxbxxxxxxcxxxxxxd"                                     , "a.**b.**c.**d" , true)
   test_case ("axxxxxxxbxxxxxxxcxxxxxxxd"                                  , "a.**b.**c.**d" , true)
   test_case ("axxxxxxxxbxxxxxxxxcxxxxxxxxd"                               , "a.**b.**c.**d" , true)
   test_case ("axxxxxxxxxbxxxxxxxxxcxxxxxxxxxd"                            , "a.**b.**c.**d" , true)
   test_case ("axxxxxxxxxxbxxxxxxxxxxcxxxxxxxxxxd"                         , "a.**b.**c.**d" , true)
   test_case ("axxxxxxxxxxxbxxxxxxxxxxxcxxxxxxxxxxxd"                      , "a.**b.**c.**d" , true)
   test_case ("axxxxxxxxxxxxbxxxxxxxxxxxxcxxxxxxxxxxxxd"                   , "a.**b.**c.**d" , true)
   test_case ("axxxxxxxxxxxxxbxxxxxxxxxxxxxcxxxxxxxxxxxxxd"                , "a.**b.**c.**d" , true)
   test_case ("axxxxxxxxxxxxxxbxxxxxxxxxxxxxxcxxxxxxxxxxxxxxd"             , "a.**b.**c.**d" , true)
   test_case ("axxxxxxxxxxxxxxxbxxxxxxxxxxxxxxxcxxxxxxxxxxxxxxxd"          , "a.**b.**c.**d" , true)
   test_case ("axxxxxxxxxxxxxxxxbxxxxxxxxxxxxxxxxcxxxxxxxxxxxxxxxxd"       , "a.**b.**c.**d" , true)
   test_case ("axxxxxxxxxxxxxxxxxbxxxxxxxxxxxxxxxxxcxxxxxxxxxxxxxxxxxd"    , "a.**b.**c.**d" , true)
   test_case ("axxxxxxxxxxxxxxxxxxbxxxxxxxxxxxxxxxxxxcxxxxxxxxxxxxxxxxxxd" , "a.**b.**c.**d" , true)
   test_case ("axbxcxd"                                                    , "a**.b**.c**.d" , true)
   test_case ("axxbxxcxxd"                                                 , "a**.b**.c**.d" , true)
   test_case ("axxxbxxxcxxxd"                                              , "a**.b**.c**.d" , true)
   test_case ("axxxxbxxxxcxxxxd"                                           , "a**.b**.c**.d" , true)
   test_case ("axxxxxbxxxxxcxxxxxd"                                        , "a**.b**.c**.d" , true)
   test_case ("axxxxxxbxxxxxxcxxxxxxd"                                     , "a**.b**.c**.d" , true)
   test_case ("axxxxxxxbxxxxxxxcxxxxxxxd"                                  , "a**.b**.c**.d" , true)
   test_case ("axxxxxxxxbxxxxxxxxcxxxxxxxxd"                               , "a**.b**.c**.d" , true)
   test_case ("axxxxxxxxxbxxxxxxxxxcxxxxxxxxxd"                            , "a**.b**.c**.d" , true)
   test_case ("axxxxxxxxxxbxxxxxxxxxxcxxxxxxxxxxd"                         , "a**.b**.c**.d" , true)
   test_case ("axxxxxxxxxxxbxxxxxxxxxxxcxxxxxxxxxxxd"                      , "a**.b**.c**.d" , true)
   test_case ("axxxxxxxxxxxxbxxxxxxxxxxxxcxxxxxxxxxxxxd"                   , "a**.b**.c**.d" , true)
   test_case ("axxxxxxxxxxxxxbxxxxxxxxxxxxxcxxxxxxxxxxxxxd"                , "a**.b**.c**.d" , true)
   test_case ("axxxxxxxxxxxxxxbxxxxxxxxxxxxxxcxxxxxxxxxxxxxxd"             , "a**.b**.c**.d" , true)
   test_case ("axxxxxxxxxxxxxxxbxxxxxxxxxxxxxxxcxxxxxxxxxxxxxxxd"          , "a**.b**.c**.d" , true)
   test_case ("axxxxxxxxxxxxxxxxbxxxxxxxxxxxxxxxxcxxxxxxxxxxxxxxxxd"       , "a**.b**.c**.d" , true)
   test_case ("axxxxxxxxxxxxxxxxxbxxxxxxxxxxxxxxxxxcxxxxxxxxxxxxxxxxxd"    , "a**.b**.c**.d" , true)
   test_case ("axxxxxxxxxxxxxxxxxxbxxxxxxxxxxxxxxxxxxcxxxxxxxxxxxxxxxxxxd" , "a**.b**.c**.d" , true)
   test_case ("axbxcxd"                                                    , "a.*b.*c.*d"                                                    , true)
   test_case ("axxbxxcxxd"                                                 , "a..*b..*c..*d"                                                 , true)
   test_case ("axxxbxxxcxxxd"                                              , "a...*b...*c...*d"                                              , true)
   test_case ("axxxxbxxxxcxxxxd"                                           , "a....*b....*c....*d"                                           , true)
   test_case ("axxxxxbxxxxxcxxxxxd"                                        , "a.....*b.....*c.....*d"                                        , true)
   test_case ("axxxxxxbxxxxxxcxxxxxxd"                                     , "a......*b......*c......*d"                                     , true)
   test_case ("axxxxxxxbxxxxxxxcxxxxxxxd"                                  , "a.......*b.......*c.......*d"                                  , true)
   test_case ("axxxxxxxxbxxxxxxxxcxxxxxxxxd"                               , "a........*b........*c........*d"                               , true)
   test_case ("axxxxxxxxxbxxxxxxxxxcxxxxxxxxxd"                            , "a.........*b.........*c.........*d"                            , true)
   test_case ("axxxxxxxxxxbxxxxxxxxxxcxxxxxxxxxxd"                         , "a..........*b..........*c..........*d"                         , true)
   test_case ("axxxxxxxxxxxbxxxxxxxxxxxcxxxxxxxxxxxd"                      , "a...........*b...........*c...........*d"                      , true)
   test_case ("axxxxxxxxxxxxbxxxxxxxxxxxxcxxxxxxxxxxxxd"                   , "a............*b............*c............*d"                   , true)
   test_case ("axxxxxxxxxxxxxbxxxxxxxxxxxxxcxxxxxxxxxxxxxd"                , "a.............*b.............*c.............*d"                , true)
   test_case ("axxxxxxxxxxxxxxbxxxxxxxxxxxxxxcxxxxxxxxxxxxxxd"             , "a..............*b..............*c..............*d"             , true)
   test_case ("axxxxxxxxxxxxxxxbxxxxxxxxxxxxxxxcxxxxxxxxxxxxxxxd"          , "a...............*b...............*c...............*d"          , true)
   test_case ("axxxxxxxxxxxxxxxxbxxxxxxxxxxxxxxxxcxxxxxxxxxxxxxxxxd"       , "a................*b................*c................*d"       , true)
   test_case ("axxxxxxxxxxxxxxxxxbxxxxxxxxxxxxxxxxxcxxxxxxxxxxxxxxxxxd"    , "a.................*b.................*c.................*d"    , true)
   test_case ("axxxxxxxxxxxxxxxxxxbxxxxxxxxxxxxxxxxxxcxxxxxxxxxxxxxxxxxxd" , "a..................*b..................*c..................*d" , true)
   test_case ("axbxcxd"                                                    , "a*.b*.c*.d"                                                    , true)
   test_case ("axxbxxcxxd"                                                 , "a*..b*..c*..d"                                                 , true)
   test_case ("axxxbxxxcxxxd"                                              , "a*...b*...c*...d"                                              , true)
   test_case ("axxxxbxxxxcxxxxd"                                           , "a*....b*....c*....d"                                           , true)
   test_case ("axxxxxbxxxxxcxxxxxd"                                        , "a*.....b*.....c*.....d"                                        , true)
   test_case ("axxxxxxbxxxxxxcxxxxxxd"                                     , "a*......b*......c*......d"                                     , true)
   test_case ("axxxxxxxbxxxxxxxcxxxxxxxd"                                  , "a*.......b*.......c*.......d"                                  , true)
   test_case ("axxxxxxxxbxxxxxxxxcxxxxxxxxd"                               , "a*........b*........c*........d"                               , true)
   test_case ("axxxxxxxxxbxxxxxxxxxcxxxxxxxxxd"                            , "a*.........b*.........c*.........d"                            , true)
   test_case ("axxxxxxxxxxbxxxxxxxxxxcxxxxxxxxxxd"                         , "a*..........b*..........c*..........d"                         , true)
   test_case ("axxxxxxxxxxxbxxxxxxxxxxxcxxxxxxxxxxxd"                      , "a*...........b*...........c*...........d"                      , true)
   test_case ("axxxxxxxxxxxxbxxxxxxxxxxxxcxxxxxxxxxxxxd"                   , "a*............b*............c*............d"                   , true)
   test_case ("axxxxxxxxxxxxxbxxxxxxxxxxxxxcxxxxxxxxxxxxxd"                , "a*.............b*.............c*.............d"                , true)
   test_case ("axxxxxxxxxxxxxxbxxxxxxxxxxxxxxcxxxxxxxxxxxxxxd"             , "a*..............b*..............c*..............d"             , true)
   test_case ("axxxxxxxxxxxxxxxbxxxxxxxxxxxxxxxcxxxxxxxxxxxxxxxd"          , "a*...............b*...............c*...............d"          , true)
   test_case ("axxxxxxxxxxxxxxxxbxxxxxxxxxxxxxxxxcxxxxxxxxxxxxxxxxd"       , "a*................b*................c*................d"       , true)
   test_case ("axxxxxxxxxxxxxxxxxbxxxxxxxxxxxxxxxxxcxxxxxxxxxxxxxxxxxd"    , "a*.................b*.................c*.................d"    , true)
   test_case ("axxxxxxxxxxxxxxxxxxbxxxxxxxxxxxxxxxxxxcxxxxxxxxxxxxxxxxxxd" , "a*..................b*..................c*..................d" , true)
   
   test_case ("a"           , "a."        , false)
   test_case ("abcd"        , "....."     , false)
   test_case ("abcd"        , "a...."     , false)
   test_case ("abcd"        , "ab..."     , false)
   test_case ("abcd"        , "abc.."     , false)
   test_case ("abcd"        , "abcd."     , false)
   test_case ("aabbbbabb"   , ".aa.*abb." , false)
   test_case ("abc"         , "xyzw"      , false)
   test_case ("abcd"        , "a."        , false)
   test_case ("abcd"        , "ab."       , false)
   test_case ("abcd"        , "xyz"       , false)
   test_case ("abcd"        , "xyzw"      , false)
   test_case ("acdcb"       , "a*c.b"     , false)
   test_case ("abcxbcdx"    , "*abcd*"    , false)
   test_case ("xabcxbcdx"   , "*abcd*"    , false)
   test_case ("a"           , "*.*."      , false)
   test_case ("a"           , "*.*.*"     , false)
   test_case ("a"           , "*.*.*."    , false)
   test_case ("a"           , ".*.*"      , false)
   test_case ("a"           , ".*.*."     , false)
   test_case ("a"           , ".*.*.*"    , false)
   
   if (all_valid)
   {
      printf("SUCCESS\n");
   }
   
   return 0;
}
